@@ -4,9 +4,11 @@ use crate::manifest::Name;
 use crate::run::Project;
 use crate::rustflags;
 use serde_derive::Deserialize;
+use std::fs::File;
 use std::path::PathBuf;
 use std::process::{Command, Output, Stdio};
-use std::{env, fs, iter};
+use std::{env, io, iter};
+use target_triple::TARGET;
 
 #[derive(Deserialize)]
 pub(crate) struct Metadata {
@@ -41,7 +43,11 @@ fn cargo(project: &Project) -> Command {
     cmd.env_remove("RUSTFLAGS");
     cmd.env("CARGO_INCREMENTAL", "0");
     cmd.arg("--offline");
-    cmd.arg(format!("--config=build.rustflags={}", rustflags::toml()));
+
+    let rustflags = rustflags::toml();
+    cmd.arg(format!("--config=build.rustflags={rustflags}"));
+    cmd.arg(format!("--config=target.{TARGET}.rustflags={rustflags}"));
+
     cmd
 }
 
@@ -66,11 +72,20 @@ pub(crate) fn manifest_dir() -> Result<Directory> {
 }
 
 pub(crate) fn build_dependencies(project: &mut Project) -> Result<()> {
-    let workspace_cargo_lock = path!(project.workspace / "Cargo.lock");
-    if workspace_cargo_lock.exists() {
-        let _ = fs::copy(workspace_cargo_lock, path!(project.dir / "Cargo.lock"));
-    } else {
-        let _ = cargo(project).arg("generate-lockfile").status();
+    // Try copying or generating lockfile.
+    match File::open(path!(project.workspace / "Cargo.lock")) {
+        Ok(mut workspace_cargo_lock) => {
+            if let Ok(mut new_cargo_lock) = File::create(path!(project.dir / "Cargo.lock")) {
+                // Not fs::copy in order to avoid producing a read-only destination
+                // file if the source file happens to be read-only.
+                let _ = io::copy(&mut workspace_cargo_lock, &mut new_cargo_lock);
+            }
+        }
+        Err(err) => {
+            if err.kind() == io::ErrorKind::NotFound {
+                let _ = cargo(project).arg("generate-lockfile").status();
+            }
+        }
     }
 
     let mut command = cargo(project);
@@ -183,12 +198,6 @@ fn features(project: &Project) -> Vec<String> {
 }
 
 fn target() -> Vec<&'static str> {
-    #[cfg(not(host_os = "windows"))]
-    const TARGET: Option<&str> = include!(concat!(env!("OUT_DIR"), "/target"));
-
-    #[cfg(host_os = "windows")]
-    const TARGET: Option<&str> = include!(concat!(env!("OUT_DIR"), "\\target"));
-
     // When --target flag is passed, cargo does not pass RUSTFLAGS to rustc when
     // building proc-macro and build script even if the host and target triples
     // are the same. Therefore, if we always pass --target to cargo, tools such
@@ -204,9 +213,7 @@ fn target() -> Vec<&'static str> {
     // Therefore, expose a cfg to always treat the target as host.
     if cfg!(trybuild_no_target) {
         vec![]
-    } else if let Some(target) = TARGET {
-        vec!["--target", target]
     } else {
-        vec![]
+        vec!["--target", TARGET]
     }
 }
