@@ -6,6 +6,7 @@ use self::Normalization::*;
 use crate::directory::Directory;
 use crate::run::PathDependency;
 use std::cmp;
+use std::mem;
 use std::path::Path;
 
 #[derive(Copy, Clone)]
@@ -59,6 +60,8 @@ normalizations! {
     StripLongTypeNameFiles,
     UnindentAfterHelp,
     AndOthersVerbose,
+    UnindentMultilineNote,
+    DependencyVersion,
     // New normalization steps are to be inserted here at the end so that any
     // snapshots saved before your normalization change remain passing.
 }
@@ -286,7 +289,7 @@ impl<'a> Filter<'a> {
                 } else if line[indent + 4..].starts_with("/rustc/")
                     && line
                         .get(indent + 11..indent + 51)
-                        .map_or(false, is_ascii_lowercase_hex)
+                        .is_some_and(is_ascii_lowercase_hex)
                     && line[indent + 51..].starts_with("/library/")
                 {
                     // --> /rustc/c5c7d2b37780dac1092e75f12ab97dd56c30861e/library/std/src/net/ip.rs:83:1
@@ -304,13 +307,27 @@ impl<'a> Filter<'a> {
                     let hash_end = hash_start + 16;
                     if line
                         .get(hash_start..hash_end)
-                        .map_or(false, is_ascii_lowercase_hex)
+                        .is_some_and(is_ascii_lowercase_hex)
                         && line[hash_end..].starts_with('/')
                     {
                         // --> /home/.cargo/registry/src/github.com-1ecc6299db9ec823/serde_json-1.0.64/src/de.rs:2584:8
                         // --> $CARGO/serde_json-1.0.64/src/de.rs:2584:8
                         line.replace_range(indent + 4..hash_end, "$CARGO");
                         other_crate = true;
+                        if self.normalization >= DependencyVersion {
+                            let rest = &line[indent + 11..];
+                            let end_of_version = rest.find('/');
+                            if let Some(end_of_crate_name) = end_of_version
+                                .and_then(|end| rest[..end].find('.'))
+                                .and_then(|end| rest[..end].rfind('-'))
+                            {
+                                line.replace_range(
+                                    indent + end_of_crate_name + 12
+                                        ..indent + end_of_version.unwrap() + 11,
+                                    "$VERSION",
+                                );
+                            }
+                        }
                     }
                 }
             }
@@ -455,7 +472,10 @@ fn is_ascii_lowercase_hex(s: &str) -> bool {
 
 // "10 | T: Send,"  ->  "   | T: Send,"
 fn hide_leading_numbers(line: &mut String) {
-    let n = line.bytes().take_while(u8::is_ascii_digit).count();
+    let n = line
+        .bytes()
+        .take_while(|b: &u8| *b == b' ' || b.is_ascii_digit())
+        .count();
     for i in 0..n {
         line.replace_range(i..i + 1, " ");
     }
@@ -542,7 +562,7 @@ fn unindent(diag: String, normalization: Normalization) -> String {
         normalized.push_str(line);
         normalized.push('\n');
 
-        if indented_line_kind(line, normalization) != IndentedLineKind::Heading {
+        if indented_line_kind(line, &mut false, normalization) != IndentedLineKind::Heading {
             continue;
         }
 
@@ -551,12 +571,15 @@ fn unindent(diag: String, normalization: Normalization) -> String {
             continue;
         };
 
-        if let IndentedLineKind::Code(indent) = indented_line_kind(next_line, normalization) {
+        if let IndentedLineKind::Code(indent) =
+            indented_line_kind(next_line, &mut false, normalization)
+        {
             if next_line[indent + 1..].starts_with("--> ") {
                 let mut lines_in_block = 1;
                 let mut least_indent = indent;
+                let mut previous_line_is_note = false;
                 while let Some(line) = ahead.next() {
-                    match indented_line_kind(line, normalization) {
+                    match indented_line_kind(line, &mut previous_line_is_note, normalization) {
                         IndentedLineKind::Heading => break,
                         IndentedLineKind::Code(indent) => {
                             lines_in_block += 1;
@@ -572,10 +595,11 @@ fn unindent(diag: String, normalization: Normalization) -> String {
                         }
                     }
                 }
+                previous_line_is_note = false;
                 for _ in 0..lines_in_block {
                     let line = lines.next().unwrap();
                     if let IndentedLineKind::Code(_) | IndentedLineKind::Other(_) =
-                        indented_line_kind(line, normalization)
+                        indented_line_kind(line, &mut previous_line_is_note, normalization)
                     {
                         let space = line.find(' ').unwrap();
                         normalized.push_str(&line[..space]);
@@ -592,7 +616,13 @@ fn unindent(diag: String, normalization: Normalization) -> String {
     normalized
 }
 
-fn indented_line_kind(line: &str, normalization: Normalization) -> IndentedLineKind {
+fn indented_line_kind(
+    line: &str,
+    previous_line_is_note: &mut bool,
+    normalization: Normalization,
+) -> IndentedLineKind {
+    let previous_line_was_note = mem::replace(previous_line_is_note, false);
+
     if let Some(heading_len) = if line.starts_with("error") {
         Some("error".len())
     } else if line.starts_with("warning") {
@@ -608,7 +638,11 @@ fn indented_line_kind(line: &str, normalization: Normalization) -> IndentedLineK
     if line.starts_with("note:")
         || line == "..."
         || normalization >= UnindentAfterHelp && line.starts_with("help:")
+        || normalization >= UnindentMultilineNote
+            && previous_line_was_note
+            && line.starts_with("      ")
     {
+        *previous_line_is_note = true;
         return IndentedLineKind::Note;
     }
 
@@ -618,8 +652,12 @@ fn indented_line_kind(line: &str, normalization: Normalization) -> IndentedLineK
         return IndentedLineKind::Code(spaces);
     }
 
-    let digits = line.bytes().take_while(u8::is_ascii_digit).count();
-    let spaces = line[digits..].bytes().take_while(|b| *b == b' ').count();
+    let mut spaces = line.bytes().take_while(is_space).count();
+    let digits = line[spaces..]
+        .bytes()
+        .take_while(u8::is_ascii_digit)
+        .count();
+    spaces += line[spaces + digits..].bytes().take_while(is_space).count();
     let rest = &line[digits + spaces..];
     if spaces > 0
         && (rest == "|"
