@@ -19,7 +19,7 @@ error[E0277]: the trait bound `Thread: serde::de::Deserialize<'_>` is not satisf
 2 |     let _ = serde_json::from_str::<std::thread::Thread>(\"???\");
   |                                    ^^^^^^^^^^^^^^^^^^^ the trait `serde::de::Deserialize<'_>` is not implemented for `Thread`
   |
- ::: $CARGO/serde_json-1.0.64/src/de.rs
+ ::: $CARGO/serde_json-$VERSION/src/de.rs
   |
   |     T: de::Deserialize<'a>,
   |        ------------------- required by this bound in `serde_json::from_str`
